'''  
    version 1.5

    Supporting information for the Geophysical Research Letters paper (2015):

    Inverting Interpolated Receiver Functions with Surface-Wave Dispersion and Gravity: 
          Application to the western US and adjacent Canada and Mexico
        Chengping Chai, Charles J. Ammon, Monica Maceira, Robert B. Herrmann.

    Interpolate velocities and/or densities for a given latitude and longitude from the 
    final (or initial) model. Or interpolate velocity and/or density for a given latitude, 
    longitude and depth. The latidue and longitude ranges are 25.5 to 54.5 N and 97.5 
    to 126.5 W, which is checked in the script. The model files can be found in the supplements
    of the above paper.

    Model files:
       2015GL063733-ds01.txt (Initial model)
       2015GL063733-ds02.txt (Inverted model)

    To use this script, please rename this file to parameter_at_point.py first.

    usage: python parameter_at_point.py [-h] -f FILENAME -t PARAMETER_TYPE -lat LAT -lon
                             LON [-dep DEPTH]

    Interpolate velocity/density from the initial/final model.

    optional arguments:
      -h, --help         show this help message and exit
      -f FILENAME        Filename of the model
      -t PARAMETER_TYPE  vs, vp, rho or all
      -lat LAT           Latitude in decimal degrees, +N.
      -lon LON           Longitude in decimal degrees, +E.
      -dep DEPTH         Depth in km.

    For example:

    	python parameter_at_point.py -f 2015GL063733-ds01.txt -t vp -lat 35.5 -lon -115

    Last update:
    	Chengping Chai 2015/01/25
    '''
#====================================================================
def vel_at_depth(depth, tops, profile):
    '''
    version 1.0

    interpolate perameters (vp, vs or rho) at a given depth for a 1D profile.
    '''
    import sys
    if depth in tops:
        depth_index = tops.index(depth)
        data1 = profile[depth_index]
    elif depth >= min(tops) and depth <= max(tops):
        i = 0
        depth0 = tops[i]
        depth2 = tops[i+1]
        while (depth0-depth)*(depth2-depth) > 0:
            i = i + 1
            depth0 = tops[i]
            depth2 = tops[i+1]
        #
        idep0 = tops.index(depth0)
        idep2 = tops.index(depth2)
        #
        data0 = profile[idep0]
        data2 = profile[idep2]
        data1 = (depth2 - depth)/(depth2-depth0)*data0 + \
                (depth - depth0)/(depth2-depth0)*data2
    else:
        data1 = False
        sys.exit('Depth input error!')
    return data1
#====================================================================
def vel_at_latlon(point_lat,point_lon,prd_data):
    '''
    version 1.0

    Compute parameters (vp, vs or rho) at a given point using bilinear 
    interpolation for a 3D model.
    '''
    import sys
    latmax = 54.5
    lonmin = -126.5
    nlat = 30
    nlon = 30
    dlat = 1.0
    dlon = 1.0
    flag = True
    if point_lat > latmax or point_lat < latmax - nlat*dlat:
        flag = False
        sys.exit('Latitude out of range!')
    if point_lon < lonmin or point_lon > lonmin + nlon*dlon:
        flag = False
        sys.exit('Longitude out of range!')
    if flag:
        ilat0 = int((latmax-point_lat)//dlat)
        ilat2 = ilat0 + 1
        lat0 = latmax - ilat0 * dlat
        lat2 = latmax - ilat2 * dlat
        ilon0 = int((point_lon-lonmin)//dlon)
        ilon2 = ilon0 + 1
        lon0 = lonmin + ilon0 * dlon
        lon2 = lonmin + ilon2 * dlon
        if lat0 == point_lat:
            data0 = prd_data[ilat0][ilon0]
            data2 = prd_data[ilat0][ilon2]
            data11 = [0 for i in range(len(data0))]
            for i in range(len(data11)):
                data11[i] = (lon2-point_lon)/(lon2-lon0)*data0[i] + \
                            (point_lon-lon0)/(lon2-lon0)*data2[i]
        elif lon0 == point_lon:
            data0 = prd_data[ilat0][ilon0]
            data2 = prd_data[ilat2][ilon0]
            data11 = [0 for i in range(len(data0))]
            for i in range(len(data11)):
                data11[i] = (point_lat-lat0)/(lat2-lat0)*data0[i] + \
                            (lat2-point_lat)/(lat2-lat0)*data2[i]
        else:
            data00 = prd_data[ilat0][ilon0]
            data02 = prd_data[ilat0][ilon2]
            data20 = prd_data[ilat2][ilon0]
            data22 = prd_data[ilat2][ilon2]
            data01 = [0 for i in range(len(data00))]
            data21 = [0 for i in range(len(data00))]
            data11 = [0 for i in range(len(data00))]
            for i in range(len(data01)):
                data01[i] = (lon2-point_lon)/(lon2-lon0)*data00[i] + \
                            (point_lon-lon0)/(lon2-lon0)*data02[i]
            for i in range(len(data21)):
                data21[i] = (lon2-point_lon)/(lon2-lon0)*data20[i] + \
                            (point_lon-lon0)/(lon2-lon0)*data22[i]
            for i in range(len(data11)):
                data11[i] = (point_lat-lat0)/(lat2-lat0)*data01[i] + \
                            (lat2-point_lat)/(lat2-lat0)*data21[i]
    else:
        data11 = flag
    return data11
#====================================================================
def read3doutputModel(modelfile):
    '''
    version 1.1

    Parse 3d model from the text file.

    '''
    fid = open(modelfile,'r')
    nx = 30
    ny = 30
    nz = 99
    prd_vp = [[[0 for k in xrange(nz)] for j in xrange(ny)] for i in xrange(nx)]
    prd_vs = [[[0 for k in xrange(nz)] for j in xrange(ny)] for i in xrange(nx)]
    prd_thicks = [[[0 for k in xrange(nz)] for j in xrange(ny)] for i in xrange(nx)]
    prd_rho = [[[0 for k in xrange(nz)] for j in xrange(ny)] for i in xrange(nx)]
    prd_lons = [[[0 for k in xrange(nz)] for j in xrange(ny)] for i in xrange(nx)]
    prd_lats = [[[0 for k in xrange(nz)] for j in xrange(ny)] for i in xrange(nx)]
    prd_smooth = [[[0 for k in xrange(nz)] for j in xrange(ny)] for i in xrange(nx)]
    prd_vsap = [[[0 for k in xrange(nz)] for j in xrange(ny)] for i in xrange(nx)]
    prd_weight = [[[0 for k in xrange(nz)] for j in xrange(ny)] for i in xrange(nx)]
    prd_ap = [[[0 for k in xrange(nz)] for j in xrange(ny)] for i in xrange(nx)]
    prd_ae = [[[0 for k in xrange(nz)] for j in xrange(ny)] for i in xrange(nx)]
    prd_tops = [[[0 for k in xrange(nz)] for j in xrange(ny)] for i in xrange(nx)]
    prd_celln = [[[0 for k in xrange(nz)] for j in xrange(ny)] for i in xrange(nx)]
    prd_geo = [[[0 for k in xrange(nz)] for j in xrange(ny)] for i in xrange(nx)]
    for ilat in range(nx):
        for ilon in range(ny):
            line1 = fid.readline()
            words = line1.split()
            lat = words[10]
            lon = words[12]
            geo = words[-1]
            celln = words[5]
            line2 = fid.readline()
            words = line2.split()
            nlay = int(words[0])
            for ilay in range(nlay):
                line = fid.readline()
                words = line.split()
                prd_vp[ilat][ilon][ilay] = float(words[1])
                prd_vs[ilat][ilon][ilay] = float(words[2])
                prd_rho[ilat][ilon][ilay] = float(words[3])
                prd_thicks[ilat][ilon][ilay] = float(words[4])
                prd_smooth[ilat][ilon][ilay] = float(words[5])
                prd_vsap[ilat][ilon][ilay] = float(words[6])
                prd_weight[ilat][ilon][ilay] = float(words[7])
                prd_ap[ilat][ilon][ilay] = float(words[8])
                prd_ae[ilat][ilon][ilay] = float(words[9])
                prd_lons[ilat][ilon][ilay] = float(lon)
                prd_lats[ilat][ilon][ilay] = float(lat)
                prd_tops[ilat][ilon][ilay] = float(words[10])
                prd_celln[ilat][ilon][ilay] = celln
                prd_geo[ilat][ilon][ilay] = geo
    fid.close()
    result = [prd_vp, prd_vs, prd_rho, prd_thicks,prd_smooth,prd_vsap,prd_weight,\
prd_ap,prd_ae,prd_lons,prd_lats, prd_tops, prd_celln, prd_geo]
    return result
#====================================================================
if __name__ == '__main__':
    
    import sys
    import argparse
    #
    parser = argparse.ArgumentParser(\
        description='Interpolate velocity/density from the initial/final model.')
    parser.add_argument('-f',dest='filename',type=str,\
        help='Filename of the model', required=True)
    parser.add_argument('-t',dest='parameter_type',type=str,\
        help='vs, vp, rho or all', required=True)
    parser.add_argument('-lat', dest='lat',type=float,\
        help='Latitude in decimal degrees, +N.', required=True)
    parser.add_argument('-lon', dest='lon',type=float,\
        help='Longitude in decimal degrees, +E.', required=True)
    parser.add_argument('-dep', dest='depth',type=float,\
        help='Depth in km.', required=False)
    args = parser.parse_args()
    #
    modelfile = args.filename
    pa_type = args.parameter_type
    point_lat = args.lat
    point_lon = args.lon
    point_depth = args.depth
    #
    if point_lon > 180:
        point_lon = point_lon - 180.0

    # read data
    result = read3doutputModel(modelfile)
    prd_vp, prd_vs, prd_rho, prd_thicks,prd_smooth,prd_vsap,prd_weight,\
    prd_ap,prd_ae,prd_lons,prd_lats, prd_tops, prd_celln, prd_geo = result
    tops = prd_tops[0][0]
    #
    if point_depth == None:
        if pa_type in ['vs','vp','rho']:
            if pa_type == 'vs':
                prd_data = prd_vs
            elif pa_type == 'vp':
                prd_data = prd_vp
            elif pa_type == 'rho':
                prd_data = prd_rho 
            data11 = vel_at_latlon(point_lat,point_lon,prd_data)
            if pa_type in ['vs','vp']:
            	print 'layer top(km), '+pa_type+' (km/s)'
            	for i in range(len(data11)):
                	print '{0:10.4f}  {1:10.4f}'.format(tops[i], data11[i])
            elif pa_type in ['rho']:
            	print 'layer top(km), '+pa_type+' (g/cm3)'
            	for i in range(len(data11)):
                	print '{0:10.4f}  {1:10.4f}'.format(tops[i], data11[i])
        elif pa_type == 'all':
            vp11 = vel_at_latlon(point_lat,point_lon,prd_vp)
            vs11 = vel_at_latlon(point_lat,point_lon,prd_vs)
            rho11 = vel_at_latlon(point_lat,point_lon,prd_rho)
            print 'layer top(km)  vp(km/s)    vs(km/s)    rho(g/cm3)'
            for i in range(len(vp11)):
                print '{0:10.4f}  {1:10.4f}  {2:10.4f}  {3:10.4f}'.format(tops[i],\
                    vp11[i],vs11[i],rho11[i])
        else:
            sys.exit('PARAMETER_TYPE is not right, choose from vp, vs, rho or all.')

    else:
        if pa_type in ['vs','vp','rho']:
            if pa_type == 'vs':
                prd_data = prd_vs
            elif pa_type == 'vp':
                prd_data = prd_vp
            elif pa_type == 'rho':
                prd_data = prd_rho 
            profile11 = vel_at_latlon(point_lat,point_lon,prd_data)
            data11 = vel_at_depth(point_depth,tops,profile11)
            print 'layer top(km), '+pa_type+' (km/s)'
            print '{0:10.4f}  {1:10.4f}'.format(point_depth, data11)
        elif pa_type == 'all':
            vp_profile11 = vel_at_latlon(point_lat,point_lon,prd_vp)
            vp11 = vel_at_depth(point_depth,tops,vp_profile11)
            vs_profile11 = vel_at_latlon(point_lat,point_lon,prd_vs)
            vs11 = vel_at_depth(point_depth,tops,vs_profile11)
            rho_profile11 = vel_at_latlon(point_lat,point_lon,prd_rho)
            rho11 = vel_at_depth(point_depth,tops,rho_profile11)
            print 'layer top(km)  vp(km/s)    vs(km/s)    rho(g/cm3)'
            print '{0:10.4f}  {1:10.4f}  {2:10.4f}  {3:10.4f}'.format(point_depth,\
                vp11,vs11,rho11)
        else:
            sys.exit('PARAMETER_TYPE is not right, choose from vp, vs, rho or all.')
